@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2013
+ * Copyright (c) 2012, 2013, 2014, 2020, 2021, 2024
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: shmemconfig.c,v 1.49 2013/10/27 15:24:42 karls Exp $";
+"$Id: shmemconfig.c,v 1.49.4.5.6.6.4.2 2024/11/20 22:05:40 karls Exp $";
 
 static size_t
 linkedsize(const linkedname_t *head);
@@ -69,11 +69,13 @@ getmem(const size_t size);
  * or NULL if there is not enough memory available.
  */
 
+# if 0
 static size_t
 memleft(void);
 /*
  * Returns the number of bytes left in the memorypool.
  */
+#endif
 
 static size_t
 memleft_no_alignment_luck(void);
@@ -130,7 +132,7 @@ do {                                                                           \
 
 #define DOCOPY(src, srcoffset, dst, attr, attrsize, memfunc)                   \
 do {                                                                           \
-   const size_t tocopy = attrsize;                                             \
+   const size_t tocopy = (attrsize);                                           \
                                                                                \
    if (tocopy == 0) {                                                          \
       SASSERTX((src)->attr == NULL);                                           \
@@ -140,7 +142,7 @@ do {                                                                           \
       SASSERTX((src)->attr != NULL);                                           \
                                                                                \
       if (((dst)->attr = memfunc(tocopy)) == NULL) {                           \
-         swarnx("%s: failed to allocate %lu byte%s of memory at line %d.  "    \
+         swarn("%s: failed to allocate %lu byte%s of memory at line %d.  "     \
                 "Memory left: %lu",                                            \
                 function,                                                      \
                 (unsigned long)tocopy,                                         \
@@ -230,6 +232,13 @@ do {                                                                           \
              dst,                                                              \
              attr.filenov,                                                     \
              sizeof(*src->attr.filenov) * src->attr.filenoc,                   \
+             memfunc);                                                         \
+                                                                               \
+      DOCOPY(src,                                                              \
+             srcoffset,                                                        \
+             dst,                                                              \
+             attr.createdv,                                                    \
+             sizeof(*src->attr.createdv) * src->attr.filenoc,                  \
              memfunc);                                                         \
    }                                                                           \
 } while (/* CONSTCOND */ 0)
@@ -251,7 +260,7 @@ pointer_copy(src, srcoffset, dst, mem, memsize)
    void *mem;
    const size_t memsize;
 {
-   const char *function = "pointer_copy()";
+/*   const char *function = "pointer_copy()"; */
 
    SASSERTX(dst != NULL);
 
@@ -316,7 +325,7 @@ pointer_copyorsize(op, src, srcoffset, dst, mem, memsize)
     * as the local saying goes.
     */
 
-    size = 0;
+   size = 0;
 
    if (src->internal.addrc > 0) {
       switch (op) {
@@ -439,19 +448,61 @@ pointer_copyorsize(op, src, srcoffset, dst, mem, memsize)
                DOLINKCOPY(srcrule,
                           srcoffset,
                           dstrule,
-                          state.ldap.ldapurl,
+                          state.ldapauthorisation.ldapurl,
                           memfunc);
 
                DOLINKCOPY(srcrule,
                           srcoffset,
                           dstrule,
-                          state.ldap.ldapbasedn,
+                          state.ldapauthorisation.ldapbasedn,
                           memfunc);
+
+
+               DOLINKCOPY(srcrule,
+                          srcoffset,
+                          dstrule,
+                          state.ldapauthorisation.ldapserver,
+                          memfunc);
+
+               DOLINKCOPY(srcrule,
+                          srcoffset,
+                          dstrule,
+                          state.ldapauthentication.ldapurl,
+                          memfunc);
+
+               DOLINKCOPY(srcrule,
+                          srcoffset,
+                          dstrule,
+                          state.ldapauthentication.ldapbasedn,
+                          memfunc);
+
+               DOLINKCOPY(srcrule,
+                          srcoffset,
+                          dstrule,
+                          state.ldapauthentication.ldapserver,
+                          memfunc);
+
                break;
 
             case SIZE:
-               ADDLEN(linkedsize(srcrule->state.ldap.ldapurl), &size);
-               ADDLEN(linkedsize(srcrule->state.ldap.ldapbasedn), &size);
+               ADDLEN(linkedsize(srcrule->state.ldapauthorisation.ldapurl),
+                      &size);
+
+               ADDLEN(linkedsize(srcrule->state.ldapauthorisation.ldapbasedn),
+                      &size);
+
+               ADDLEN(linkedsize(srcrule->state.ldapauthorisation.ldapserver),
+                      &size);
+
+               ADDLEN(linkedsize(srcrule->state.ldapauthentication.ldapurl),
+                      &size);
+
+               ADDLEN(linkedsize(srcrule->state.ldapauthentication.ldapbasedn),
+                      &size);
+
+               ADDLEN(linkedsize(srcrule->state.ldapauthentication.ldapserver),
+                      &size);
+
                break;
          }
 
@@ -473,12 +524,10 @@ pointer_copyorsize(op, src, srcoffset, dst, mem, memsize)
          switch (op) {
             case COPY:
                DOLINKCOPY(srcrule, srcoffset, dstrule, ldapgroup, memfunc);
-               DOLINKCOPY(srcrule, srcoffset, dstrule, ldapserver, memfunc);
                break;
 
             case SIZE:
                ADDLEN(linkedsize(srcrule->ldapgroup), &size);
-               ADDLEN(linkedsize(srcrule->ldapserver), &size);
                break;
          }
 #endif /* HAVE_LDAP */
@@ -495,9 +544,9 @@ pointer_copyorsize(op, src, srcoffset, dst, mem, memsize)
                          sizeof(*srcrule->next),
                          memfunc);
 
-                  srcrule = srcrule->next;
-                  dstrule = dstrule->next;
-                  break;
+               srcrule = srcrule->next;
+               dstrule = dstrule->next;
+               break;
 
             case SIZE:
                if (srcrule->next != NULL)
@@ -652,8 +701,9 @@ pointer_copyorsize(op, src, srcoffset, dst, mem, memsize)
             if (logv[i]->filenoc > 0) {
                size_t ii;
 
-               ADDLEN(sizeof(*(logv[i]->filenov)) * logv[i]->filenoc, &size);
-               ADDLEN(sizeof((*logv[i]->fnamev))  * logv[i]->filenoc, &size);
+               ADDLEN(sizeof((*logv[i]->fnamev))   * logv[i]->filenoc, &size);
+               ADDLEN(sizeof(*(logv[i]->createdv)) * logv[i]->filenoc, &size);
+               ADDLEN(sizeof(*(logv[i]->filenov))  * logv[i]->filenoc, &size);
 
                for (ii = 0; ii < logv[i]->filenoc; ++ii)
                   ADDLEN(strlen(logv[i]->fnamev[ii]) + 1, &size);
@@ -774,17 +824,23 @@ do {                                                                           \
    size_t i, compared = 0;
 
    /*
-    * Check all pointers and structs, as that's where an error is likely*
+    * Check all pointers and structs, as that's where an error is likely
     * to occur.  Also check non-pointers where easy to do, though it should
     * be almost impossible for an error to exists there due to the initial
     * struct assignment between 'a' and 'b'.
     */
 
    EQCHECK(a, b, initial);
-   EQCHECK_PTR(a, b, internal.addrv, sizeof(*a->internal.addrv));
+
    EQCHECK_PTR(a,
                b,
-               external.addrv, sizeof(*a->external.addrv) * a->external.addrc);
+               internal.addrv,
+               sizeof(*a->internal.addrv) * a->internal.addrc);
+
+   EQCHECK_PTR(a,
+               b,
+               external.addrv,
+               sizeof(*a->external.addrv) * a->external.addrc);
    EQCHECK(a, b, cpu);
 
    PTRCHECK(a, b, crule);
@@ -828,15 +884,39 @@ do {                                                                           \
          tmpruleb = *brule;
 
 #if HAVE_LDAP
-         tmprulea.state.ldap.ldapurl    = tmpruleb.state.ldap.ldapurl;
-         tmprulea.state.ldap.ldapbasedn = tmpruleb.state.ldap.ldapbasedn;
+
+         tmprulea.state.ldapauthorisation.ldapurl
+         = tmpruleb.state.ldapauthorisation.ldapurl;
+
+         tmprulea.state.ldapauthorisation.ldapbasedn
+         = tmpruleb.state.ldapauthorisation.ldapbasedn;
+
+         tmprulea.state.ldapauthorisation.ldapserver
+         = tmpruleb.state.ldapauthorisation.ldapserver;
+
+         tmprulea.state.ldapauthentication.ldapurl
+         = tmpruleb.state.ldapauthentication.ldapurl;
+
+         tmprulea.state.ldapauthentication.ldapbasedn 
+         = tmpruleb.state.ldapauthentication.ldapbasedn;
+
+         tmprulea.state.ldapauthentication.ldapserver 
+         = tmpruleb.state.ldapauthentication.ldapserver;
+
 #endif /* HAVE_LDAP */
 
          EQCHECK((&tmprulea), (&tmpruleb), state);
 
 #if HAVE_LDAP
-         LINKCHECK(arule, brule, state.ldap.ldapurl);
-         LINKCHECK(arule, brule, state.ldap.ldapbasedn);
+
+         LINKCHECK(arule, brule, state.ldapauthorisation.ldapurl);
+         LINKCHECK(arule, brule, state.ldapauthorisation.ldapbasedn);
+         LINKCHECK(arule, brule, state.ldapauthorisation.ldapserver);
+         LINKCHECK(arule, brule, state.ldapauthentication.ldapurl);
+         LINKCHECK(arule, brule, state.ldapauthentication.ldapbasedn);
+         LINKCHECK(arule, brule, state.ldapauthentication.ldapserver);
+
+
 #endif /* HAVE_LDAP */
 
          EQCHECK(arule, brule, timeout);
@@ -844,13 +924,16 @@ do {                                                                           \
          LINKCHECK(arule, brule, group);
 
 #if HAVE_LDAP
+
          LINKCHECK(arule, brule, ldapgroup);
-         LINKCHECK(arule, brule, ldapserver);
          EQCHECK(arule, brule, ldapsettingsfromuser);
+
 #endif /* HAVE_LDAP */
 
 #if HAVE_LIBWRAP
+
          EQCHECK(arule, brule, libwrap);
+
 #endif /* HAVE_LIBWRAP */
 
          EQCHECK(arule, brule, bw_shmid);
@@ -926,6 +1009,8 @@ do {                                                                           \
       EQCHECK(alog, blog, type);
 
       PTRCHECK(alog, blog, filenov);
+      PTRCHECK(alog, blog, createdv);
+
       if (alog->fnamev != NULL) {
          size_t ii;
 
@@ -940,7 +1025,14 @@ do {                                                                           \
                      blog,
                      filenov,
                      sizeof(*alog->filenov) * alog->filenoc);
+
+         SASSERTX(alog->createdv != NULL);
+         EQCHECK_PTR(alog,
+                     blog,
+                     createdv,
+                     sizeof(*alog->createdv) * alog->filenoc);
       }
+
       PTRCHECK(alog, blog, fnamev);
 
       EQCHECK(alog, blog, filenoc);
@@ -1083,7 +1175,7 @@ doconfigtest(void)
                     0x0,
                     shmem,
                     (char *)shmem + sizeof(sockscf),
-                    size - sizeof(sockscf)) == 0)
+                    size          - sizeof(sockscf)) == 0)
       slog(LOG_DEBUG, "%s: pointer_copy() successful", function);
    else
       serr("%s: pointer_copy() failed", function);
@@ -1188,11 +1280,13 @@ doconfigtest(void)
    /* free(shmem).  Should free shmem->pointers too, but can not. */
 }
 
+#if 0
 static size_t
 memleft(void)
 {
    return _left;
 }
+#endif
 
 static size_t
 memleft_no_alignment_luck(void)
