@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
- *               2008, 2009, 2010, 2011, 2012, 2013, 2014
+ *               2008, 2009, 2010, 2011, 2012, 2013, 2014, 2016, 2017, 2019,
+ *               2020, 2021, 2024
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +46,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: sockd.c,v 1.925.4.5 2014/08/21 13:17:13 karls Exp $";
+"$Id: sockd.c,v 1.925.4.5.2.4.4.10.4.6 2024/12/02 20:46:57 karls Exp $";
 
 
 
@@ -69,7 +70,8 @@ serverinit(int argc, char *argv[]);
  */
 
 static void
-usage(int code);
+usage(int code)
+      __ATTRIBUTE__((noreturn));
 /*
  * print usage.
  */
@@ -81,7 +83,8 @@ showversion(const int level);
  */
 
 static void
-showlicense(void);
+showlicense(void)
+            __ATTRIBUTE__((noreturn));
 /*
  * shows license and exits.
  */
@@ -150,7 +153,7 @@ main(argc, argv)
        have_saved_io     = 0;
 
 #if DIAGNOSTIC && HAVE_MALLOC_OPTIONS
-   malloc_options = "AFGJP";
+   malloc_options = "S";
 #endif /* DIAGNOSTIC && HAVE_MALLOC_OPTIONS */
 
 #if ELECTRICFENCE
@@ -503,8 +506,8 @@ moncontrol(1);
                clientinfo_t cinfo;
 
                if (SHMID_ISSET(CRULE_OR_HRULE(&saved_req))) {
-                  cinfo.from = saved_req.from;
-                  HOSTIDCOPY(&saved_req.state, &cinfo);
+                  cinfo.from   = saved_req.from;
+                  cinfo.hostid = saved_req.state.hostid;
                }
 
                if (socketisconnected(saved_req.s, NULL, 0) == NULL) {
@@ -593,8 +596,8 @@ moncontrol(1);
                clientinfo_t cinfo;
 
                if (SHMID_ISSET(&saved_io.srule)) {
-                  cinfo.from = CONTROLIO(&saved_io)->raddr;
-                  HOSTIDCOPY(&saved_io.state, &cinfo);
+                  cinfo.from   = CONTROLIO(&saved_io)->raddr;
+                  cinfo.hostid = saved_io.state.hostid;
                }
 
                if (socketisconnected(CONTROLIO(&saved_io)->s, NULL, 0) == NULL){
@@ -672,7 +675,6 @@ moncontrol(1);
           * returned above points to.
           */
          const sockd_child_t fromchild = *child;
-         unsigned char command;
          int childhasfinished;
 
          if (sockd_handledsignals())
@@ -730,7 +732,10 @@ moncontrol(1);
                }
 
                ++sockscf.stat.negotiate.received;
-               command = req.reqinfo.command;
+
+               handlechildcommand(req.reqinfo.command,
+                                  getchild(fromchild.pid),
+                                  &childhasfinished);
 
                log_clientsend(&req.from, child, 0);
 
@@ -754,8 +759,8 @@ moncontrol(1);
                   clientinfo_t cinfo;
 
                   if (SHMID_ISSET(CRULE_OR_HRULE(&req))) {
-                     cinfo.from = req.from;
-                     HOSTIDCOPY(&req.state, &cinfo);
+                     cinfo.from   = req.from;
+                     cinfo.hostid = req.state.hostid;
                   }
 
                   if (!socketisconnected(req.s, NULL, 0)) {
@@ -855,7 +860,10 @@ moncontrol(1);
                }
 
                ++sockscf.stat.request.received;
-               command = io.reqinfo.command;
+
+               handlechildcommand(io.reqinfo.command,
+                                  getchild(fromchild.pid),
+                                  &childhasfinished);
 
                switch (io.state.protocol) {
                   case SOCKS_TCP:
@@ -911,8 +919,8 @@ moncontrol(1);
                else {
                   clientinfo_t cinfo;
                   if (SHMID_ISSET(&io.srule)) {
-                     cinfo.from = CONTROLIO(&io)->raddr;
-                     HOSTIDCOPY(&io.state, &cinfo);
+                     cinfo.from   = CONTROLIO(&io)->raddr;
+                     cinfo.hostid = io.state.hostid;
                   }
 
                   if (socketisconnected(CONTROLIO(&io)->s, NULL, 0) == NULL) {
@@ -928,7 +936,7 @@ moncontrol(1);
                   else {
                      if (io.state.command != SOCKS_UDPASSOCIATE
                      &&  io.dst.s         != -1
-                     && !socketisconnected(io.dst.s, NULL, 0) != 0) {
+                     && socketisconnected(io.dst.s, NULL, 0)) {
                         if (io.srule.mstats_shmid != 0
                         && (io.srule.alarmsconfigured & ALARM_DISCONNECT))
                            alarm_add_disconnect(0,
@@ -1003,7 +1011,11 @@ moncontrol(1);
                }
 
                ++sockscf.stat.io.received;
-               command = client.reqinfo.command;
+
+               handlechildcommand(client.reqinfo.command,
+                                  getchild(fromchild.pid),
+                                  &childhasfinished);
+
 
                log_clientsend(&client.from, child, 0);
                p = send_client(child->s, &client, NULL, 0);
@@ -1061,10 +1073,6 @@ moncontrol(1);
                SERRX(fromchild.type);
          }
 
-         handlechildcommand(command,
-                            getchild(fromchild.pid),
-                            &childhasfinished);
-
          if (childhasfinished)
             closechild(fromchild.pid, 1);
       }
@@ -1154,6 +1162,21 @@ moncontrol(1);
                      continue; /* check if there are more connections pending */
                }
 
+               if (IPADDRISBOUND(&sockscf.internal.addrv[i].addr))
+                  client.to = sockscf.internal.addrv[i].addr;
+               else {
+                  /*
+                   * wildcard address.  Don't know what address client was
+                   * accepted on.
+                   */
+                  socklen_t len = sizeof(client.to);
+
+                  if (getsockname(client.s, TOSA(&client.to), &len) != 0) {
+                     slog(LOG_DEBUG, "getsockname(2) failed after accept(2)");
+                     continue; /* check if there are more connections pending */
+                  }
+               }
+
                gettimeofday_monotonic(&client.accepted);
                ++sockscf.stat.accepted;
 
@@ -1209,7 +1232,7 @@ usage(code)
 {
 
    (void)fprintf(code == 0 ? stdout : stderr,
-"%s v%s.  Copyright (c) 1997 - 2014, Inferno Nettverk A/S, Norway.\n"
+"%s v%s.  Copyright (c) 1997 - 2024, Inferno Nettverk A/S, Norway.\n"
 "usage: %s [-DLNVdfhnv]\n"
 "   -D             : run in daemon mode\n"
 "   -L             : shows the license for this program\n"
@@ -1245,6 +1268,12 @@ extern const size_t       module_ldap_keyc;
 extern const char         module_ldap_version[];
 #endif /* HAVE_LDAP */
 
+#if HAVE_PAC
+extern const licensekey_t module_pac_keyv[];
+extern const size_t       module_pac_keyc;
+extern const char         module_pac_version[];
+#endif /* HAVE_PAC */
+
 static void
 showversion(level)
    const int level;
@@ -1267,6 +1296,13 @@ showversion(level)
                            },
 #endif /* HAVE_LDAP */
 
+#if HAVE_PAC
+                           {  "PAC",
+                              module_pac_keyv,
+                              module_pac_keyc
+                           },
+#endif /* HAVE_PAC */
+
                            {  "Redirect",
                               module_redirect_keyv,
                               module_redirect_keyc
@@ -1276,7 +1312,7 @@ showversion(level)
 
    size_t i;
 
-   printf("%s v%s.  Copyright (c) 1997 - 2014 Inferno Nettverk A/S, Norway\n",
+   printf("%s v%s.  Copyright (c) 1997 - 2024 Inferno Nettverk A/S, Norway\n",
           PRODUCT, VERSION);
 
    for (i = 0; i < licenseinfoc; ++i) {
@@ -1328,7 +1364,8 @@ showlicense(void)
 "\
 /*\n\
  * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,\n\
- *               2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014\n\
+ *               2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,\n\
+ *               2017, 2018, 2019, 2020, 2021, 2024\n\
  *      Inferno Nettverk A/S, Norway.  All rights reserved.\n\
  *\n\
  * Redistribution and use in source and binary forms, with or without\n\
@@ -1361,7 +1398,7 @@ showlicense(void)
  *  Software Distribution Coordinator  or  sdc@inet.no\n\
  *  Inferno Nettverk A/S\n\
  *  Oslo Research Park\n\
- *  Gaustadallen 21\n\
+ *  Gaustadalleen 21\n\
  *  NO-0349 Oslo\n\
  *  Norway\n\
  * \n\
@@ -1379,6 +1416,7 @@ serverinit(argc, argv)
    char *argv[];
 {
    const char *function = "serverinit()";
+   size_t i;
    int ch;
 
 #if !HAVE_PROGNAME
@@ -1406,6 +1444,9 @@ serverinit(argc, argv)
    sockscf.shmemconfigfd  = -1;
    sockscf.shmemfd        = -1;
 
+   for (i = 0; i < ELEMENTS(sockscf.state.reservedfdv); ++i)
+      sockscf.state.reservedfdv[i] = -1;
+
    while ((ch = getopt(argc, argv, "DLN:Vd:f:hnp:v")) != -1) {
       switch (ch) {
          case 'D':
@@ -1613,16 +1654,18 @@ handlechildcommand(command, child, finished)
          SWARNX(command);
    }
 
-   if (sockscf.child.maxrequests != 0
-   &&  child->freec              == maxfreeslots(child->type)
-   &&  child->sentc              >= sockscf.child.maxrequests) {
+   if (child->freec == maxfreeslots(child->type)
+   &&  child_should_retire(child)) {
+      time_t tnow;
+
       slog(LOG_DEBUG,
-           "%s: should close  connection to %s %ld: %lu request%s handled",
+           "%s: %s %ld is ready for retirement: %lu requests handled "
+           "during %lu seconds",
            function,
            childtype2string(child->type),
            (long)child->pid,
            (unsigned long)child->sentc,
-           (unsigned long)child->sentc == 1 ? "" : "s");
+           socks_difftime(time_monotonic(&tnow), child->created));
 
       *finished = 1;
    }
@@ -1634,7 +1677,7 @@ handlechildcommand(command, child, finished)
 static void
 dotest(void)
 {
-   const char *function = "dotest()";
+/*   const char *function = "dotest()"; */
 
    doconfigtest();
 
